/*
 * Created by wangshengye on 2020/05/07.
 */

#ifndef ELFSIGN_ELF_64_H
#define ELFSIGN_ELF_64_H
#ifdef __cplusplus
extern "C" {
#endif

#include <elf.h>
#include <stdio.h>
#include <string.h>
#include <stdbool.h>
#include <openssl/sha.h>

typedef struct {
    long int size;
    char *path;
    Elf64_Ehdr ehdr;
    Elf64_Shdr shstrtabhdr;
    char *shstrtab;
    //unsigned char digest[SHA_DIGEST_LENGTH];
    unsigned char sign[512];
    unsigned char *cert;
    long int cert_size;
    char *section_text;
    int  section_text_size;
} Elf64;
int DeleteDataElf64(const char *fileName, int deletePos,int dataSize);

int InsertDataElf64(const char *fileName, int insertPos, const char * insertData,int dataSize);

bool IsELF64(const char *file);

bool HasSignELF64(Elf64 *elf64);

void SetElf64Path(Elf64 *elf64, const char *path);

bool GetEhdr64(Elf64 *elf64);

bool Getshstrtabhdr64(Elf64 *elf64);

bool Getshstrtab64(Elf64 *elf64);

int GetFileSize64(Elf64 *elf64);

bool AddSectionHeader64(Elf64 *elf64, Elf64_Word shSize);

bool DeleteSectionHeader64(Elf64 *elf64);

bool CreateSignSection64(Elf64 *elf64, Elf64_Shdr *signSection, Elf64_Word shSize);

bool AddSectionData64(Elf64 *elf64, const char *sectionData, int dataSize);

bool DeleteSectionData64(Elf64 *elf64,  int dataSize);

bool AddSectionName64(Elf64 *elf64, const char *seName, int nameSize);

bool DeleteSectionName64(Elf64 *elf64, int nameSize);

bool UpdateShstrtabSize64(Elf64 *elf64, int increaseSize);

bool DeupdateShstrtabSize64(Elf64 *elf64, int deleteSize); 

bool UpdateShstrtabOffset64(Elf64 *elf64, int increaseOffset);

bool DeupdateShstrtabOffset64(Elf64 *elf64, int deleteOffset);

bool UpdateShnum64(Elf64 *elf64);

bool DeupdateShnum64(Elf64 *elf64);

bool UpdateShoffset64(Elf64 *elf64, Elf64_Off increaseSize);

bool DeupdateShoffset64(Elf64 *elf64, Elf64_Off deleteSize);

bool UpdateShstrndex64(Elf64 *elf64);

bool DeupdateShstrndex64(Elf64 *elf64);

bool FindSectionOffsetSize(Elf64 *elf64, const char *seName, Elf64_Off *sectionOffset, Elf64_Xword *sectionSize);

bool HashText64(Elf64 *elf64);

void Destract64(Elf64 *elf64);
#ifdef __cplusplus
}
#endif

#endif //ELFSIGN_ELF_64_H
